// MSP430_MORSE.c
// Wandelt einen Text in (evtl. mehrere) Long-Words fr die vereinfachte Morseausgabe mit dem MSP430 um.
// 11.01.2016 DL9MWE

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define PauseLang 5 // Pause zwischen Worten (ca. 6 dits)


typedef struct {
        unsigned int anzahl_elemente;   // dits und dahs zhlen je eins
        unsigned long bitmuster;        // LSB first
        unsigned int pausenzeit;        // 3 zwischen Buchstaben, 7 zwischen Worten
        unsigned char teiltext[17];     // max. 16 'e's
} bitmuster_t;


typedef struct {
    unsigned char zeichen;
    unsigned char* morse_code;
} morse_codes_t;


#define anzahl_zeichen 42
static const morse_codes_t morse_codes[anzahl_zeichen] = {
    'A', ".-",
    'B', "-...",
    'C', "-.-.",
    'D', "-..",
    'E', ".",
    'F', "..-.",
    'G', "--.",
    'H', "....",
    'I', "..",
    'J', ".---",
    'K', "-.-",
    'L', ".-..",
    'M', "--",
    'N', "-.",
    'O', "---",
    'P', ".--.",
    'Q', "--.-",
    'R', ".-.",
    'S', "...",
    'T', "-",
    'U', "..-",
    'V', "...-",
    'W', ".--",
    'X', "-..-",
    'Y', "-.--",
    'Z', "--..",

    '0', "-----",
    '1', ".----",
    '2', "..---",
    '3', "...--",
    '4', "....-",
    '5', ".....",
    '6', "-....",
    '7', "--...",
    '8', "---..",
    '9', "----.",
    
    '?', "..--..",
    '=', "-...-",
    '/', "-..-.",
    '+', ".-.-.",
    '.', ".-.-.-",
    '0', "........" // Irrung fr fehlende Zeichen..
};
 
 
// Buchstaben/Zeichen in der Tabelle suchen
unsigned int search_zeichen(unsigned char zeichen, unsigned char *morse_code) {
    unsigned int index_liste=0;
    unsigned int element_count;
    for(index_liste=0; index_liste < anzahl_zeichen-1; ++index_liste) {
        if(morse_codes[index_liste].zeichen == toupper(zeichen)) {
            break;
        }
    } // entweder das Zeichen wurde gefunden, oder eine Irrung wird gegeben
    strcpy(morse_code, morse_codes[index_liste].morse_code);
    for(element_count=0; morse_code[element_count]; ++element_count);
    return element_count;
}

// Bitmuster fr ein Zeichen erzeugen
unsigned long calculate_bitmuster(char *morse_code) {
    unsigned int index;
    unsigned long muster=0;
    for(index = 0; morse_code[index]; ++index) {
        muster >>= 2;
        if(morse_code[index] == '-') {
            muster |= 0x80000000;
        }
    }
    muster >>= 1;
    muster |= 0x80000000;
    return muster;
}

unsigned int calculate(unsigned char text[], bitmuster_t *muster) {
    unsigned int index_text=0, index_teiltext=0;
    unsigned int anzahl_elemente=0, elemente;
    unsigned int anzahl_aufrufe=0;
    unsigned char morse_code[9];
    
    muster[anzahl_aufrufe].bitmuster = 0;
    muster[anzahl_aufrufe].pausenzeit = 0;
    while(text[index_text]) {
        if(text[index_text] == ' ') {
            muster[anzahl_aufrufe].pausenzeit = PauseLang;
            muster[anzahl_aufrufe].bitmuster >>= 32-anzahl_elemente*2;
            anzahl_elemente = 0;
            ++anzahl_aufrufe;
            muster[anzahl_aufrufe].bitmuster = 0;
            muster[anzahl_aufrufe].pausenzeit = 0;
            muster[anzahl_aufrufe].teiltext[index_teiltext = 0] = '\0';;
            ++index_text;
            continue;
        }
        elemente = search_zeichen(text[index_text], morse_code);
        if(anzahl_elemente+elemente > 16) {
            muster[anzahl_aufrufe].bitmuster >>= 32-anzahl_elemente*2;
            ++anzahl_aufrufe;
            anzahl_elemente = 0;
            muster[anzahl_aufrufe].bitmuster = 0;
            muster[anzahl_aufrufe].pausenzeit = 0;
            index_teiltext = 0;
        }
        muster[anzahl_aufrufe].teiltext[index_teiltext] = toupper(text[index_text]);
        muster[anzahl_aufrufe].teiltext[++index_teiltext] = '\0';
        anzahl_elemente += elemente;
        muster[anzahl_aufrufe].anzahl_elemente = anzahl_elemente;
        muster[anzahl_aufrufe].bitmuster >>= elemente*2;
        muster[anzahl_aufrufe].bitmuster |= calculate_bitmuster(morse_code);
        ++index_text;
    }

    muster[anzahl_aufrufe].bitmuster >>= 32-anzahl_elemente*2;
    muster[anzahl_aufrufe].pausenzeit = PauseLang;
    return anzahl_aufrufe;
}


int main(int argc, char *argv[])
{
    char text[101];            // sollte reichen...
    unsigned int anzahl_aufrufe;
    bitmuster_t muster[50];    // ebenfalls mehr als genug...
    unsigned int index;

    printf("Umrechnung eines Eingabetextes (<=100 Zeichen) in Programmcode\n");
    printf(" fuer den Aufruf der Routine\n vMorseBeeper(unsigned int count, unsigned long bitmuster, unsigned int pausenzeit);\n\n");
       
    while(42) {
        char filename[105];     // sollte auch reichen
        FILE *outfile;
        
        printf("Eingabetext (<100 Zeichen) ?\n");
        fgets(text, 100, stdin);
        text[strlen(text)-1] = '\0';

        sprintf(filename, "%s.txt", text);  // Ausgabedatei ffnen
        if((outfile = fopen(filename, "w")) == NULL) {
            printf("Ausgabedatei '%s' kann nicht erstellt werden\n", filename);
        }
        anzahl_aufrufe = calculate(text, &muster);
        printf("Anzahl Aufrufe von vMorseBeeper: %i\n", anzahl_aufrufe+1);
        for(index=0; index <= anzahl_aufrufe; ++index) {
            printf("\tvMorseBeeper(2*%i, 0x%08xL, %d*MS_KURZ);\t// %s\n", muster[index].anzahl_elemente, muster[index].bitmuster, muster[index].pausenzeit, muster[index].teiltext);
            if(outfile != NULL)
                fprintf(outfile, "\tvMorseBeeper(2*%i, 0x%08xL, %d*MS_KURZ);\t// %s\n", muster[index].anzahl_elemente, muster[index].bitmuster, muster[index].pausenzeit, muster[index].teiltext);
        }
        fclose(outfile);
    }
}

// Ende MSP430_MORSE.c

